/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        12. May 2016
 * $Revision:    V1.00
 *  
 * Project:      Flash Programming Functions for Toshiba TMPM3Hx Flashes
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */ 

#include "..\FlashOS.H"        // FlashOS Structures

#define M8(adr)  (*((volatile unsigned char  *)(adr)))
#define M16(adr) (*((volatile unsigned short *)(adr)))
#define M32(adr) (*((volatile unsigned long  *)(adr)))

#define STACK_SIZE     64      // Stack Size
#define PAGE_SIZE    4096      // Page Size for TMPM343 but data is written in junks of 4 Words


// Watchdog Timer Registers
#define SIWD0EN          (*((volatile unsigned long  *)0x400BB404))
#define SIWD0CR          (*((volatile unsigned long  *)0x400BB408))

// Protect Status Register 0
#define FCKCR            (*((volatile unsigned long  *)0x5DFF0018))
#define FCSR0            (*((volatile unsigned long  *)0x5DFF0020))
#define FCSR0_RDY        (1ul << 0)

// Area Select Register
#define FCAREASEL        (*((volatile unsigned long  *)0x5DFF0140))
#define FCAREASEL_AREA4  (7ul << 16)


unsigned long base_adr;
unsigned long area_adr;
unsigned long area_sel;

/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  // Disable Watchdog
  SIWD0EN = 0x00000000;
  SIWD0CR = 0x000000B1;

  base_adr = adr;

  return (0);
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  return (0);
}


/*  Blank Check Block in Flash Memory
 *    Parameter:      adr:  Block Start Address
 *                    sz:   Block Size (in bytes)
 *                    pat:  Block Pattern
 *    Return Value:   0 - OK,  1 - Failed
 */

int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat) {
  return (1);                                    // Always Force Erase
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {

	area_adr = 0x30000000ul;
	area_sel = FCAREASEL_AREA4;

	FCKCR     = 0xa74a9d23ul;
	FCAREASEL = area_sel;                          // Specify the target area of Flash memory
	while(!(FCAREASEL & 0x40000000));
	while(!(FCSR0 & 0x00000001));

	// Start Chip Erase Command
	M32(area_adr | 0x00000550) = 0x000000AA;
	M32(area_adr | 0x00000AA0) = 0x00000055;
	M32(area_adr | 0x00000550) = 0x00000080;
	M32(area_adr | 0x00000550) = 0x000000AA;
	M32(area_adr | 0x00000AA0) = 0x00000055;
	M32(area_adr             ) = 0x00000020;

	while(FCSR0 & 0x00000001);
	while(!(FCSR0 & 0x00000001));

	FCKCR     = 0xa74a9d23;
	FCAREASEL = 0x00000000;
	while(FCAREASEL & 0x40000000);

	return (0);                                    // Finished without Errors
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {

	area_adr = 0x30000000ul;
	area_sel = FCAREASEL_AREA4;

	FCKCR     = 0xa74a9d23ul;
	FCAREASEL = area_sel;                          // Specify the target area of Flash memory
	while(!(FCAREASEL & 0x40000000));
	while(!(FCSR0 & 0x00000001));

	// Start Block Erase Command
	M32(area_adr | 0x00000550) = 0x000000AA;
	M32(area_adr | 0x00000AA0) = 0x00000055;
	M32(area_adr | 0x00000550) = 0x00000080;
	M32(area_adr | 0x00000550) = 0x000000AA;
	M32(area_adr | 0x00000AA0) = 0x00000055;
	M32(area_adr | adr       ) = 0x00000040;

	while(FCSR0 & 0x00000001);
	while(!(FCSR0 & 0x00000001));

	FCKCR     = 0xa74a9d23;
	FCAREASEL = 0x00000000;
	while(FCAREASEL & 0x40000000);

	return (0);                                    // Finished without Errors
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
	int len = sz;

	area_adr = 0x30000000ul;
	area_sel = FCAREASEL_AREA4;

	FCKCR     = 0xa74a9d23ul;
	FCAREASEL = area_sel;                          // Specify the target area of Flash memory
	while(!(FCAREASEL & 0x40000000));
	while(!(FCSR0 & 0x00000001));

	while (len > 0) {
		// Start Page Programming Command
		M32(area_adr | 0x00000550) = 0x000000AA;
		M32(area_adr | 0x00000AA0) = 0x00000055;
		M32(area_adr | 0x00000550) = 0x000000C0;

		// Write Data in junks of 4 * 4 words
		M32(area_adr | adr) = *((unsigned long *)(buf));
		adr += 4;
		buf += 4;
		len -= 4;

		while(FCSR0 & 0x00000001);
		while(!(FCSR0 & 0x00000001));
	}

	FCKCR     = 0xa74a9d23;
	FCAREASEL = 0x00000000;
	while(FCAREASEL & 0x40000000);

	return (0);                                    // Finished without Errors
}
